from enum import Enum


class PostSubmodelElementByPathSubmodelRepoExtent(str, Enum):
    WITHBLOBVALUE = "withBlobValue"
    WITHOUTBLOBVALUE = "withoutBlobValue"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.operation_request_client_timeout_duration import OperationRequestClientTimeoutDuration
    from ..models.operation_variable import OperationVariable


T = TypeVar("T", bound="OperationRequest")


@_attrs_define
class OperationRequest:
    """Operation request object

    Attributes:
        input_arguments (Union[Unset, List['OperationVariable']]):
        inoutput_arguments (Union[Unset, List['OperationVariable']]):
        client_timeout_duration (Union[Unset, OperationRequestClientTimeoutDuration]):
    """

    input_arguments: Union[Unset, List["OperationVariable"]] = UNSET
    inoutput_arguments: Union[Unset, List["OperationVariable"]] = UNSET
    client_timeout_duration: Union[Unset, "OperationRequestClientTimeoutDuration"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        input_arguments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.input_arguments, Unset):
            input_arguments = []
            for input_arguments_item_data in self.input_arguments:
                input_arguments_item = input_arguments_item_data.to_dict()
                input_arguments.append(input_arguments_item)

        inoutput_arguments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.inoutput_arguments, Unset):
            inoutput_arguments = []
            for inoutput_arguments_item_data in self.inoutput_arguments:
                inoutput_arguments_item = inoutput_arguments_item_data.to_dict()
                inoutput_arguments.append(inoutput_arguments_item)

        client_timeout_duration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_timeout_duration, Unset):
            client_timeout_duration = self.client_timeout_duration.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if input_arguments is not UNSET:
            field_dict["inputArguments"] = input_arguments
        if inoutput_arguments is not UNSET:
            field_dict["inoutputArguments"] = inoutput_arguments
        if client_timeout_duration is not UNSET:
            field_dict["clientTimeoutDuration"] = client_timeout_duration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.operation_request_client_timeout_duration import OperationRequestClientTimeoutDuration
        from ..models.operation_variable import OperationVariable

        d = src_dict.copy()
        input_arguments = []
        _input_arguments = d.pop("inputArguments", UNSET)
        for input_arguments_item_data in _input_arguments or []:
            input_arguments_item = OperationVariable.from_dict(input_arguments_item_data)

            input_arguments.append(input_arguments_item)

        inoutput_arguments = []
        _inoutput_arguments = d.pop("inoutputArguments", UNSET)
        for inoutput_arguments_item_data in _inoutput_arguments or []:
            inoutput_arguments_item = OperationVariable.from_dict(inoutput_arguments_item_data)

            inoutput_arguments.append(inoutput_arguments_item)

        _client_timeout_duration = d.pop("clientTimeoutDuration", UNSET)
        client_timeout_duration: Union[Unset, OperationRequestClientTimeoutDuration]
        if isinstance(_client_timeout_duration, Unset):
            client_timeout_duration = UNSET
        else:
            client_timeout_duration = OperationRequestClientTimeoutDuration.from_dict(_client_timeout_duration)

        operation_request = cls(
            input_arguments=input_arguments,
            inoutput_arguments=inoutput_arguments,
            client_timeout_duration=client_timeout_duration,
        )

        operation_request.additional_properties = d
        return operation_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.result import Result
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    aas_ids: Union[Unset, List[str]] = UNSET,
    submodel_ids: Union[Unset, List[str]] = UNSET,
    include_concept_descriptions: Union[Unset, str] = "true",
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_aas_ids: Union[Unset, List[str]] = UNSET
    if not isinstance(aas_ids, Unset):
        json_aas_ids = aas_ids

    params["aasIds"] = json_aas_ids

    json_submodel_ids: Union[Unset, List[str]] = UNSET
    if not isinstance(submodel_ids, Unset):
        json_submodel_ids = submodel_ids

    params["submodelIds"] = json_submodel_ids

    params["includeConceptDescriptions"] = include_concept_descriptions

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/serialization",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Result]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Result.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Result.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Result.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Result.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Result.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = Result.from_dict(response.json())

        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Result]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    aas_ids: Union[Unset, List[str]] = UNSET,
    submodel_ids: Union[Unset, List[str]] = UNSET,
    include_concept_descriptions: Union[Unset, str] = "true",
) -> Response[Result]:
    """Returns an appropriate serialization based on the specified format (see SerializationFormat)

    Args:
        aas_ids (Union[Unset, List[str]]):
        submodel_ids (Union[Unset, List[str]]):
        include_concept_descriptions (Union[Unset, str]):  Default: 'true'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Result]
    """

    kwargs = _get_kwargs(
        aas_ids=aas_ids,
        submodel_ids=submodel_ids,
        include_concept_descriptions=include_concept_descriptions,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    aas_ids: Union[Unset, List[str]] = UNSET,
    submodel_ids: Union[Unset, List[str]] = UNSET,
    include_concept_descriptions: Union[Unset, str] = "true",
) -> Optional[Result]:
    """Returns an appropriate serialization based on the specified format (see SerializationFormat)

    Args:
        aas_ids (Union[Unset, List[str]]):
        submodel_ids (Union[Unset, List[str]]):
        include_concept_descriptions (Union[Unset, str]):  Default: 'true'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Result
    """

    return sync_detailed(
        client=client,
        aas_ids=aas_ids,
        submodel_ids=submodel_ids,
        include_concept_descriptions=include_concept_descriptions,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    aas_ids: Union[Unset, List[str]] = UNSET,
    submodel_ids: Union[Unset, List[str]] = UNSET,
    include_concept_descriptions: Union[Unset, str] = "true",
) -> Response[Result]:
    """Returns an appropriate serialization based on the specified format (see SerializationFormat)

    Args:
        aas_ids (Union[Unset, List[str]]):
        submodel_ids (Union[Unset, List[str]]):
        include_concept_descriptions (Union[Unset, str]):  Default: 'true'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Result]
    """

    kwargs = _get_kwargs(
        aas_ids=aas_ids,
        submodel_ids=submodel_ids,
        include_concept_descriptions=include_concept_descriptions,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    aas_ids: Union[Unset, List[str]] = UNSET,
    submodel_ids: Union[Unset, List[str]] = UNSET,
    include_concept_descriptions: Union[Unset, str] = "true",
) -> Optional[Result]:
    """Returns an appropriate serialization based on the specified format (see SerializationFormat)

    Args:
        aas_ids (Union[Unset, List[str]]):
        submodel_ids (Union[Unset, List[str]]):
        include_concept_descriptions (Union[Unset, str]):  Default: 'true'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Result
    """

    return (
        await asyncio_detailed(
            client=client,
            aas_ids=aas_ids,
            submodel_ids=submodel_ids,
            include_concept_descriptions=include_concept_descriptions,
        )
    ).parsed



from enum import Enum


class ReferenceType(str, Enum):
    EXTERNAL_REFERENCE = "ExternalReference"
    MODEL_REFERENCE = "ModelReference"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AssetInformationAssetKind(str, Enum):
    INSTANCE = "Instance"
    NOT_APPLICABLE = "NotApplicable"
    TYPE = "Type"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Base64UrlEncodedCursor")


@_attrs_define
class Base64UrlEncodedCursor:
    """
    Attributes:
        encoded_cursor (Union[Unset, str]):
        decoded_cursor (Union[Unset, str]):
    """

    encoded_cursor: Union[Unset, str] = UNSET
    decoded_cursor: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        encoded_cursor = self.encoded_cursor

        decoded_cursor = self.decoded_cursor

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if encoded_cursor is not UNSET:
            field_dict["encodedCursor"] = encoded_cursor
        if decoded_cursor is not UNSET:
            field_dict["decodedCursor"] = decoded_cursor

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        encoded_cursor = d.pop("encodedCursor", UNSET)

        decoded_cursor = d.pop("decodedCursor", UNSET)

        base_64_url_encoded_cursor = cls(
            encoded_cursor=encoded_cursor,
            decoded_cursor=decoded_cursor,
        )

        base_64_url_encoded_cursor.additional_properties = d
        return base_64_url_encoded_cursor

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MessageMessageType(str, Enum):
    ERROR = "ERROR"
    EXCEPTION = "EXCEPTION"
    INFO = "INFO"
    WARNING = "WARNING"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.result import Result
from ...types import Response


def _get_kwargs(
    aas_identifier: str,
    submodel_identifier: str,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "delete",
        "url": f"/shells/{aas_identifier}/submodel-refs/{submodel_identifier}",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, Result]]:
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Result.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NO_CONTENT:
        response_204 = cast(Any, None)
        return response_204
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Result.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Result.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Result.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.OK:
        response_200 = Result.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = Result.from_dict(response.json())

        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, Result]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    aas_identifier: str,
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[Any, Result]]:
    """Deletes the submodel reference from the Asset Administration Shell. Does not delete the submodel
    itself!

    Args:
        aas_identifier (str):
        submodel_identifier (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Result]]
    """

    kwargs = _get_kwargs(
        aas_identifier=aas_identifier,
        submodel_identifier=submodel_identifier,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    aas_identifier: str,
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[Any, Result]]:
    """Deletes the submodel reference from the Asset Administration Shell. Does not delete the submodel
    itself!

    Args:
        aas_identifier (str):
        submodel_identifier (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Result]
    """

    return sync_detailed(
        aas_identifier=aas_identifier,
        submodel_identifier=submodel_identifier,
        client=client,
    ).parsed


async def asyncio_detailed(
    aas_identifier: str,
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[Any, Result]]:
    """Deletes the submodel reference from the Asset Administration Shell. Does not delete the submodel
    itself!

    Args:
        aas_identifier (str):
        submodel_identifier (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Result]]
    """

    kwargs = _get_kwargs(
        aas_identifier=aas_identifier,
        submodel_identifier=submodel_identifier,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    aas_identifier: str,
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[Any, Result]]:
    """Deletes the submodel reference from the Asset Administration Shell. Does not delete the submodel
    itself!

    Args:
        aas_identifier (str):
        submodel_identifier (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Result]
    """

    return (
        await asyncio_detailed(
            aas_identifier=aas_identifier,
            submodel_identifier=submodel_identifier,
            client=client,
        )
    ).parsed



from enum import Enum


class SubmodelElementListTypeValueListElement(str, Enum):
    ANNOTATED_RELATIONSHIP_ELEMENT = "ANNOTATED_RELATIONSHIP_ELEMENT"
    BASIC_EVENT_ELEMENT = "BASIC_EVENT_ELEMENT"
    BLOB = "BLOB"
    CAPABILITY = "CAPABILITY"
    DATA_ELEMENT = "DATA_ELEMENT"
    ENTITY = "ENTITY"
    EVENT_ELEMENT = "EVENT_ELEMENT"
    FILE = "FILE"
    MULTI_LANGUAGE_PROPERTY = "MULTI_LANGUAGE_PROPERTY"
    OPERATION = "OPERATION"
    PROPERTY = "PROPERTY"
    RANGE = "RANGE"
    REFERENCE_ELEMENT = "REFERENCE_ELEMENT"
    RELATIONSHIP_ELEMENT = "RELATIONSHIP_ELEMENT"
    SUBMODEL_ELEMENT = "SUBMODEL_ELEMENT"
    SUBMODEL_ELEMENT_COLLECTION = "SUBMODEL_ELEMENT_COLLECTION"
    SUBMODEL_ELEMENT_LIST = "SUBMODEL_ELEMENT_LIST"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ServiceDescriptionProfilesItem(str, Enum):
    HTTPSADMIN_SHELL_IOAASAPI30AASXFILESERVERSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/AasxFileServerServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30ASSETADMINISTRATIONSHELLREGISTRYSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/AssetAdministrationShellRegistryServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30ASSETADMINISTRATIONSHELLREGISTRYSERVICESPECIFICATIONSSP_002 = (
        "https://admin-shell.io/aas/API/3/0/AssetAdministrationShellRegistryServiceSpecification/SSP-002"
    )
    HTTPSADMIN_SHELL_IOAASAPI30ASSETADMINISTRATIONSHELLREPOSITORYSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/AssetAdministrationShellRepositoryServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30ASSETADMINISTRATIONSHELLREPOSITORYSERVICESPECIFICATIONSSP_002 = (
        "https://admin-shell.io/aas/API/3/0/AssetAdministrationShellRepositoryServiceSpecification/SSP-002"
    )
    HTTPSADMIN_SHELL_IOAASAPI30ASSETADMINISTRATIONSHELLSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/AssetAdministrationShellServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30ASSETADMINISTRATIONSHELLSERVICESPECIFICATIONSSP_002 = (
        "https://admin-shell.io/aas/API/3/0/AssetAdministrationShellServiceSpecification/SSP-002"
    )
    HTTPSADMIN_SHELL_IOAASAPI30CONCEPTDESCRIPTIONSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/ConceptDescriptionServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30DISCOVERYSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/DiscoveryServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELREGISTRYSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelRegistryServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELREGISTRYSERVICESPECIFICATIONSSP_002 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelRegistryServiceSpecification/SSP-002"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELREPOSITORYSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelRepositoryServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELREPOSITORYSERVICESPECIFICATIONSSP_002 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelRepositoryServiceSpecification/SSP-002"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELREPOSITORYSERVICESPECIFICATIONSSP_003 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelRepositoryServiceSpecification/SSP-003"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELREPOSITORYSERVICESPECIFICATIONSSP_004 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelRepositoryServiceSpecification/SSP-004"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELSERVICESPECIFICATIONSSP_001 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelServiceSpecification/SSP-001"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELSERVICESPECIFICATIONSSP_002 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelServiceSpecification/SSP-002"
    )
    HTTPSADMIN_SHELL_IOAASAPI30SUBMODELSERVICESPECIFICATIONSSP_003 = (
        "https://admin-shell.io/aas/API/3/0/SubmodelServiceSpecification/SSP-003"
    )

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.extension_value_type import ExtensionValueType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.reference import Reference


T = TypeVar("T", bound="Extension")


@_attrs_define
class Extension:
    """
    Attributes:
        name (Union[Unset, str]):
        value (Union[Unset, str]):
        value_type (Union[Unset, ExtensionValueType]):
        refers_to (Union[Unset, List['Reference']]):
        semantic_id (Union[Unset, Reference]):
        supplemental_semantic_ids (Union[Unset, List['Reference']]):
    """

    name: Union[Unset, str] = UNSET
    value: Union[Unset, str] = UNSET
    value_type: Union[Unset, ExtensionValueType] = UNSET
    refers_to: Union[Unset, List["Reference"]] = UNSET
    semantic_id: Union[Unset, "Reference"] = UNSET
    supplemental_semantic_ids: Union[Unset, List["Reference"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        value = self.value

        value_type: Union[Unset, str] = UNSET
        if not isinstance(self.value_type, Unset):
            value_type = self.value_type.value

        refers_to: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.refers_to, Unset):
            refers_to = []
            for refers_to_item_data in self.refers_to:
                refers_to_item = refers_to_item_data.to_dict()
                refers_to.append(refers_to_item)

        semantic_id: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.semantic_id, Unset):
            semantic_id = self.semantic_id.to_dict()

        supplemental_semantic_ids: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.supplemental_semantic_ids, Unset):
            supplemental_semantic_ids = []
            for supplemental_semantic_ids_item_data in self.supplemental_semantic_ids:
                supplemental_semantic_ids_item = supplemental_semantic_ids_item_data.to_dict()
                supplemental_semantic_ids.append(supplemental_semantic_ids_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if value is not UNSET:
            field_dict["value"] = value
        if value_type is not UNSET:
            field_dict["valueType"] = value_type
        if refers_to is not UNSET:
            field_dict["refersTo"] = refers_to
        if semantic_id is not UNSET:
            field_dict["semanticId"] = semantic_id
        if supplemental_semantic_ids is not UNSET:
            field_dict["supplementalSemanticIds"] = supplemental_semantic_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.reference import Reference

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        value = d.pop("value", UNSET)

        _value_type = d.pop("valueType", UNSET)
        value_type: Union[Unset, ExtensionValueType]
        if isinstance(_value_type, Unset):
            value_type = UNSET
        else:
            value_type = ExtensionValueType(_value_type)

        refers_to = []
        _refers_to = d.pop("refersTo", UNSET)
        for refers_to_item_data in _refers_to or []:
            refers_to_item = Reference.from_dict(refers_to_item_data)

            refers_to.append(refers_to_item)

        _semantic_id = d.pop("semanticId", UNSET)
        semantic_id: Union[Unset, Reference]
        if isinstance(_semantic_id, Unset):
            semantic_id = UNSET
        else:
            semantic_id = Reference.from_dict(_semantic_id)

        supplemental_semantic_ids = []
        _supplemental_semantic_ids = d.pop("supplementalSemanticIds", UNSET)
        for supplemental_semantic_ids_item_data in _supplemental_semantic_ids or []:
            supplemental_semantic_ids_item = Reference.from_dict(supplemental_semantic_ids_item_data)

            supplemental_semantic_ids.append(supplemental_semantic_ids_item)

        extension = cls(
            name=name,
            value=value,
            value_type=value_type,
            refers_to=refers_to,
            semantic_id=semantic_id,
            supplemental_semantic_ids=supplemental_semantic_ids,
        )

        extension.additional_properties = d
        return extension

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="OperationRequestClientTimeoutDurationXmlschemaType")


@_attrs_define
class OperationRequestClientTimeoutDurationXmlschemaType:
    """
    Attributes:
        namespace_uri (Union[Unset, str]):
        local_part (Union[Unset, str]):
        prefix (Union[Unset, str]):
    """

    namespace_uri: Union[Unset, str] = UNSET
    local_part: Union[Unset, str] = UNSET
    prefix: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        namespace_uri = self.namespace_uri

        local_part = self.local_part

        prefix = self.prefix

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if namespace_uri is not UNSET:
            field_dict["namespaceURI"] = namespace_uri
        if local_part is not UNSET:
            field_dict["localPart"] = local_part
        if prefix is not UNSET:
            field_dict["prefix"] = prefix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        namespace_uri = d.pop("namespaceURI", UNSET)

        local_part = d.pop("localPart", UNSET)

        prefix = d.pop("prefix", UNSET)

        operation_request_client_timeout_duration_xmlschema_type = cls(
            namespace_uri=namespace_uri,
            local_part=local_part,
            prefix=prefix,
        )

        operation_request_client_timeout_duration_xmlschema_type.additional_properties = d
        return operation_request_client_timeout_duration_xmlschema_type

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class KeyType(str, Enum):
    ANNOTATED_RELATIONSHIP_ELEMENT = "ANNOTATED_RELATIONSHIP_ELEMENT"
    ASSET_ADMINISTRATION_SHELL = "AssetAdministrationShell"
    BASIC_EVENT_ELEMENT = "BASIC_EVENT_ELEMENT"
    BLOB = "BLOB"
    CAPABILITY = "CAPABILITY"
    CONCEPT_DESCRIPTION = "CONCEPT_DESCRIPTION"
    DATA_ELEMENT = "DATA_ELEMENT"
    ENTITY = "ENTITY"
    EVENT_ELEMENT = "EVENT_ELEMENT"
    FILE = "FILE"
    FRAGMENT_REFERENCE = "FRAGMENT_REFERENCE"
    GLOBAL_REFERENCE = "GlobalReference"
    IDENTIFIABLE = "IDENTIFIABLE"
    MULTI_LANGUAGE_PROPERTY = "MULTI_LANGUAGE_PROPERTY"
    OPERATION = "Operation"
    PROPERTY = "Property"
    RANGE = "RANGE"
    REFERABLE = "REFERABLE"
    REFERENCE_ELEMENT = "ReferenceElement"
    RELATIONSHIP_ELEMENT = "RELATIONSHIP_ELEMENT"
    SUBMODEL = "Submodel"
    SUBMODEL_ELEMENT = "SUBMODEL_ELEMENT"
    SUBMODEL_ELEMENT_COLLECTION = "SubmodelElementCollection"
    SUBMODEL_ELEMENT_LIST = "SubmodelElementList"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.administrative_information import AdministrativeInformation
    from ..models.embedded_data_specification import EmbeddedDataSpecification
    from ..models.extension import Extension
    from ..models.lang_string_name_type import LangStringNameType
    from ..models.lang_string_text_type import LangStringTextType
    from ..models.reference import Reference


T = TypeVar("T", bound="ConceptDescription")


@_attrs_define
class ConceptDescription:
    """
    Attributes:
        is_case_of (Union[Unset, List['Reference']]):
        id (Union[Unset, str]):
        administration (Union[Unset, AdministrativeInformation]):
        display_name (Union[Unset, List['LangStringNameType']]):
        description (Union[Unset, List['LangStringTextType']]):
        category (Union[Unset, str]):
        id_short (Union[Unset, str]):
        extensions (Union[Unset, List['Extension']]):
        embedded_data_specifications (Union[Unset, List['EmbeddedDataSpecification']]):
    """

    is_case_of: Union[Unset, List["Reference"]] = UNSET
    id: Union[Unset, str] = UNSET
    administration: Union[Unset, "AdministrativeInformation"] = UNSET
    display_name: Union[Unset, List["LangStringNameType"]] = UNSET
    description: Union[Unset, List["LangStringTextType"]] = UNSET
    category: Union[Unset, str] = UNSET
    id_short: Union[Unset, str] = UNSET
    extensions: Union[Unset, List["Extension"]] = UNSET
    embedded_data_specifications: Union[Unset, List["EmbeddedDataSpecification"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_case_of: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.is_case_of, Unset):
            is_case_of = []
            for is_case_of_item_data in self.is_case_of:
                is_case_of_item = is_case_of_item_data.to_dict()
                is_case_of.append(is_case_of_item)

        id = self.id

        administration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.administration, Unset):
            administration = self.administration.to_dict()

        display_name: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.display_name, Unset):
            display_name = []
            for display_name_item_data in self.display_name:
                display_name_item = display_name_item_data.to_dict()
                display_name.append(display_name_item)

        description: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.description, Unset):
            description = []
            for description_item_data in self.description:
                description_item = description_item_data.to_dict()
                description.append(description_item)

        category = self.category

        id_short = self.id_short

        extensions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.extensions, Unset):
            extensions = []
            for extensions_item_data in self.extensions:
                extensions_item = extensions_item_data.to_dict()
                extensions.append(extensions_item)

        embedded_data_specifications: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.embedded_data_specifications, Unset):
            embedded_data_specifications = []
            for embedded_data_specifications_item_data in self.embedded_data_specifications:
                embedded_data_specifications_item = embedded_data_specifications_item_data.to_dict()
                embedded_data_specifications.append(embedded_data_specifications_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_case_of is not UNSET:
            field_dict["isCaseOf"] = is_case_of
        if id is not UNSET:
            field_dict["id"] = id
        if administration is not UNSET:
            field_dict["administration"] = administration
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if description is not UNSET:
            field_dict["description"] = description
        if category is not UNSET:
            field_dict["category"] = category
        if id_short is not UNSET:
            field_dict["idShort"] = id_short
        if extensions is not UNSET:
            field_dict["extensions"] = extensions
        if embedded_data_specifications is not UNSET:
            field_dict["embeddedDataSpecifications"] = embedded_data_specifications

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.administrative_information import AdministrativeInformation
        from ..models.embedded_data_specification import EmbeddedDataSpecification
        from ..models.extension import Extension
        from ..models.lang_string_name_type import LangStringNameType
        from ..models.lang_string_text_type import LangStringTextType
        from ..models.reference import Reference

        d = src_dict.copy()
        is_case_of = []
        _is_case_of = d.pop("isCaseOf", UNSET)
        for is_case_of_item_data in _is_case_of or []:
            is_case_of_item = Reference.from_dict(is_case_of_item_data)

            is_case_of.append(is_case_of_item)

        id = d.pop("id", UNSET)

        _administration = d.pop("administration", UNSET)
        administration: Union[Unset, AdministrativeInformation]
        if isinstance(_administration, Unset):
            administration = UNSET
        else:
            administration = AdministrativeInformation.from_dict(_administration)

        display_name = []
        _display_name = d.pop("displayName", UNSET)
        for display_name_item_data in _display_name or []:
            display_name_item = LangStringNameType.from_dict(display_name_item_data)

            display_name.append(display_name_item)

        description = []
        _description = d.pop("description", UNSET)
        for description_item_data in _description or []:
            description_item = LangStringTextType.from_dict(description_item_data)

            description.append(description_item)

        category = d.pop("category", UNSET)

        id_short = d.pop("idShort", UNSET)

        extensions = []
        _extensions = d.pop("extensions", UNSET)
        for extensions_item_data in _extensions or []:
            extensions_item = Extension.from_dict(extensions_item_data)

            extensions.append(extensions_item)

        embedded_data_specifications = []
        _embedded_data_specifications = d.pop("embeddedDataSpecifications", UNSET)
        for embedded_data_specifications_item_data in _embedded_data_specifications or []:
            embedded_data_specifications_item = EmbeddedDataSpecification.from_dict(
                embedded_data_specifications_item_data
            )

            embedded_data_specifications.append(embedded_data_specifications_item)

        concept_description = cls(
            is_case_of=is_case_of,
            id=id,
            administration=administration,
            display_name=display_name,
            description=description,
            category=category,
            id_short=id_short,
            extensions=extensions,
            embedded_data_specifications=embedded_data_specifications,
        )

        concept_description.additional_properties = d
        return concept_description

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.asset_administration_shell import AssetAdministrationShell
    from ..models.paged_result_paging_metadata import PagedResultPagingMetadata


T = TypeVar("T", bound="GetAssetAdministrationShellsResult")


@_attrs_define
class GetAssetAdministrationShellsResult:
    """
    Attributes:
        paging_metadata (Union[Unset, PagedResultPagingMetadata]):
        result (Union[Unset, List['AssetAdministrationShell']]):
    """

    paging_metadata: Union[Unset, "PagedResultPagingMetadata"] = UNSET
    result: Union[Unset, List["AssetAdministrationShell"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        paging_metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.paging_metadata, Unset):
            paging_metadata = self.paging_metadata.to_dict()

        result: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.result, Unset):
            result = []
            for result_item_data in self.result:
                result_item = result_item_data.to_dict()
                result.append(result_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if paging_metadata is not UNSET:
            field_dict["paging_metadata"] = paging_metadata
        if result is not UNSET:
            field_dict["result"] = result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.asset_administration_shell import AssetAdministrationShell
        from ..models.paged_result_paging_metadata import PagedResultPagingMetadata

        d = src_dict.copy()
        _paging_metadata = d.pop("paging_metadata", UNSET)
        paging_metadata: Union[Unset, PagedResultPagingMetadata]
        if isinstance(_paging_metadata, Unset):
            paging_metadata = UNSET
        else:
            paging_metadata = PagedResultPagingMetadata.from_dict(_paging_metadata)

        result = []
        _result = d.pop("result", UNSET)
        for result_item_data in _result or []:
            result_item = AssetAdministrationShell.from_dict(result_item_data)

            result.append(result_item)

        get_asset_administration_shells_result = cls(
            paging_metadata=paging_metadata,
            result=result,
        )

        get_asset_administration_shells_result.additional_properties = d
        return get_asset_administration_shells_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.submodel_element import SubmodelElement


T = TypeVar("T", bound="OperationVariable")


@_attrs_define
class OperationVariable:
    """
    Attributes:
        value (Union[Unset, SubmodelElement]):
    """

    value: Union[Unset, "SubmodelElement"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.value, Unset):
            value = self.value.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.submodel_element import SubmodelElement

        d = src_dict.copy()
        _value = d.pop("value", UNSET)
        value: Union[Unset, SubmodelElement]
        if isinstance(_value, Unset):
            value = UNSET
        else:
            value = SubmodelElement.from_dict(_value)

        operation_variable = cls(
            value=value,
        )

        operation_variable.additional_properties = d
        return operation_variable

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EntityEntityType(str, Enum):
    CO_MANAGED_ENTITY = "CO_MANAGED_ENTITY"
    SELF_MANAGED_ENTITY = "SELF_MANAGED_ENTITY"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.entity import Entity
from ...models.operation import Operation
from ...models.patch_submodel_by_id_value_only_level import PatchSubmodelByIdValueOnlyLevel
from ...models.relationship_element import RelationshipElement
from ...models.result import Result
from ...models.submodel_element import SubmodelElement
from ...models.submodel_element_collection import SubmodelElementCollection
from ...models.submodel_element_list import SubmodelElementList
from ...models.submodel_value_only import SubmodelValueOnly
from ...types import UNSET, Response, Unset


def _get_kwargs(
    submodel_identifier: str,
    *,
    body: List[
        Union[
            "Entity",
            "Operation",
            "RelationshipElement",
            "SubmodelElement",
            "SubmodelElementCollection",
            "SubmodelElementList",
        ]
    ],
    level: Union[Unset, PatchSubmodelByIdValueOnlyLevel] = PatchSubmodelByIdValueOnlyLevel.DEEP,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    params: Dict[str, Any] = {}

    json_level: Union[Unset, str] = UNSET
    if not isinstance(level, Unset):
        json_level = level.value

    params["level"] = json_level

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "patch",
        "url": f"/submodels/{submodel_identifier}/$value",
        "params": params,
    }

    _body = []
    for body_item_data in body:
        body_item: Dict[str, Any]
        if isinstance(body_item_data, SubmodelElement):
            body_item = body_item_data.to_dict()
        elif isinstance(body_item_data, SubmodelElement):
            body_item = body_item_data.to_dict()
        elif isinstance(body_item_data, Entity):
            body_item = body_item_data.to_dict()
        elif isinstance(body_item_data, SubmodelElement):
            body_item = body_item_data.to_dict()
        elif isinstance(body_item_data, Operation):
            body_item = body_item_data.to_dict()
        elif isinstance(body_item_data, RelationshipElement):
            body_item = body_item_data.to_dict()
        elif isinstance(body_item_data, SubmodelElementCollection):
            body_item = body_item_data.to_dict()
        else:
            body_item = body_item_data.to_dict()

        _body.append(body_item)

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Result, SubmodelValueOnly]]:
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Result.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Result.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Result.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.NO_CONTENT:
        response_204 = SubmodelValueOnly.from_dict(response.json())

        return response_204
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Result.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = Result.from_dict(response.json())

        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Result, SubmodelValueOnly]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: List[
        Union[
            "Entity",
            "Operation",
            "RelationshipElement",
            "SubmodelElement",
            "SubmodelElementCollection",
            "SubmodelElementList",
        ]
    ],
    level: Union[Unset, PatchSubmodelByIdValueOnlyLevel] = PatchSubmodelByIdValueOnlyLevel.DEEP,
) -> Response[Union[Result, SubmodelValueOnly]]:
    """Updates the values of an existing Submodel

    Args:
        submodel_identifier (str):
        level (Union[Unset, PatchSubmodelByIdValueOnlyLevel]):  Default:
            PatchSubmodelByIdValueOnlyLevel.DEEP.
        body (List[Union['Entity', 'Operation', 'RelationshipElement', 'SubmodelElement',
            'SubmodelElementCollection', 'SubmodelElementList']]): Submodel object in its ValueOnly
            representation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Result, SubmodelValueOnly]]
    """

    kwargs = _get_kwargs(
        submodel_identifier=submodel_identifier,
        body=body,
        level=level,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: List[
        Union[
            "Entity",
            "Operation",
            "RelationshipElement",
            "SubmodelElement",
            "SubmodelElementCollection",
            "SubmodelElementList",
        ]
    ],
    level: Union[Unset, PatchSubmodelByIdValueOnlyLevel] = PatchSubmodelByIdValueOnlyLevel.DEEP,
) -> Optional[Union[Result, SubmodelValueOnly]]:
    """Updates the values of an existing Submodel

    Args:
        submodel_identifier (str):
        level (Union[Unset, PatchSubmodelByIdValueOnlyLevel]):  Default:
            PatchSubmodelByIdValueOnlyLevel.DEEP.
        body (List[Union['Entity', 'Operation', 'RelationshipElement', 'SubmodelElement',
            'SubmodelElementCollection', 'SubmodelElementList']]): Submodel object in its ValueOnly
            representation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Result, SubmodelValueOnly]
    """

    return sync_detailed(
        submodel_identifier=submodel_identifier,
        client=client,
        body=body,
        level=level,
    ).parsed


async def asyncio_detailed(
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: List[
        Union[
            "Entity",
            "Operation",
            "RelationshipElement",
            "SubmodelElement",
            "SubmodelElementCollection",
            "SubmodelElementList",
        ]
    ],
    level: Union[Unset, PatchSubmodelByIdValueOnlyLevel] = PatchSubmodelByIdValueOnlyLevel.DEEP,
) -> Response[Union[Result, SubmodelValueOnly]]:
    """Updates the values of an existing Submodel

    Args:
        submodel_identifier (str):
        level (Union[Unset, PatchSubmodelByIdValueOnlyLevel]):  Default:
            PatchSubmodelByIdValueOnlyLevel.DEEP.
        body (List[Union['Entity', 'Operation', 'RelationshipElement', 'SubmodelElement',
            'SubmodelElementCollection', 'SubmodelElementList']]): Submodel object in its ValueOnly
            representation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Result, SubmodelValueOnly]]
    """

    kwargs = _get_kwargs(
        submodel_identifier=submodel_identifier,
        body=body,
        level=level,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    submodel_identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: List[
        Union[
            "Entity",
            "Operation",
            "RelationshipElement",
            "SubmodelElement",
            "SubmodelElementCollection",
            "SubmodelElementList",
        ]
    ],
    level: Union[Unset, PatchSubmodelByIdValueOnlyLevel] = PatchSubmodelByIdValueOnlyLevel.DEEP,
) -> Optional[Union[Result, SubmodelValueOnly]]:
    """Updates the values of an existing Submodel

    Args:
        submodel_identifier (str):
        level (Union[Unset, PatchSubmodelByIdValueOnlyLevel]):  Default:
            PatchSubmodelByIdValueOnlyLevel.DEEP.
        body (List[Union['Entity', 'Operation', 'RelationshipElement', 'SubmodelElement',
            'SubmodelElementCollection', 'SubmodelElementList']]): Submodel object in its ValueOnly
            representation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Result, SubmodelValueOnly]
    """

    return (
        await asyncio_detailed(
            submodel_identifier=submodel_identifier,
            client=client,
            body=body,
            level=level,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.submodel_value_only_values_only_map import SubmodelValueOnlyValuesOnlyMap


T = TypeVar("T", bound="SubmodelValueOnly")


@_attrs_define
class SubmodelValueOnly:
    """
    Attributes:
        id_short (Union[Unset, str]):
        values_only_map (Union[Unset, SubmodelValueOnlyValuesOnlyMap]):
    """

    id_short: Union[Unset, str] = UNSET
    values_only_map: Union[Unset, "SubmodelValueOnlyValuesOnlyMap"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id_short = self.id_short

        values_only_map: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.values_only_map, Unset):
            values_only_map = self.values_only_map.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id_short is not UNSET:
            field_dict["idShort"] = id_short
        if values_only_map is not UNSET:
            field_dict["valuesOnlyMap"] = values_only_map

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.submodel_value_only_values_only_map import SubmodelValueOnlyValuesOnlyMap

        d = src_dict.copy()
        id_short = d.pop("idShort", UNSET)

        _values_only_map = d.pop("valuesOnlyMap", UNSET)
        values_only_map: Union[Unset, SubmodelValueOnlyValuesOnlyMap]
        if isinstance(_values_only_map, Unset):
            values_only_map = UNSET
        else:
            values_only_map = SubmodelValueOnlyValuesOnlyMap.from_dict(_values_only_map)

        submodel_value_only = cls(
            id_short=id_short,
            values_only_map=values_only_map,
        )

        submodel_value_only.additional_properties = d
        return submodel_value_only

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.reference_type import ReferenceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.key import Key


T = TypeVar("T", bound="Reference")


@_attrs_define
class Reference:
    """
    Attributes:
        type (Union[Unset, ReferenceType]):
        keys (Union[Unset, List['Key']]):
        referred_semantic_id (Union[Unset, Reference]):
    """

    type: Union[Unset, ReferenceType] = UNSET
    keys: Union[Unset, List["Key"]] = UNSET
    referred_semantic_id: Union[Unset, "Reference"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        keys: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.keys, Unset):
            keys = []
            for keys_item_data in self.keys:
                keys_item = keys_item_data.to_dict()
                keys.append(keys_item)

        referred_semantic_id: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.referred_semantic_id, Unset):
            referred_semantic_id = self.referred_semantic_id.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if keys is not UNSET:
            field_dict["keys"] = keys
        if referred_semantic_id is not UNSET:
            field_dict["referredSemanticId"] = referred_semantic_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.key import Key

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, ReferenceType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = ReferenceType(_type)

        keys = []
        _keys = d.pop("keys", UNSET)
        for keys_item_data in _keys or []:
            keys_item = Key.from_dict(keys_item_data)

            keys.append(keys_item)

        _referred_semantic_id = d.pop("referredSemanticId", UNSET)
        referred_semantic_id: Union[Unset, Reference]
        if isinstance(_referred_semantic_id, Unset):
            referred_semantic_id = UNSET
        else:
            referred_semantic_id = Reference.from_dict(_referred_semantic_id)

        reference = cls(
            type=type,
            keys=keys,
            referred_semantic_id=referred_semantic_id,
        )

        reference.additional_properties = d
        return reference

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_submodel_element_by_path_submodel_repo_extent import GetSubmodelElementByPathSubmodelRepoExtent
from ...models.get_submodel_element_by_path_submodel_repo_level import GetSubmodelElementByPathSubmodelRepoLevel
from ...models.result import Result
from ...models.submodel_element import SubmodelElement
from ...types import UNSET, Response, Unset


def _get_kwargs(
    submodel_identifier: str,
    id_short_path: str,
    *,
    level: Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel] = GetSubmodelElementByPathSubmodelRepoLevel.DEEP,
    extent: Union[
        Unset, GetSubmodelElementByPathSubmodelRepoExtent
    ] = GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_level: Union[Unset, str] = UNSET
    if not isinstance(level, Unset):
        json_level = level.value

    params["level"] = json_level

    json_extent: Union[Unset, str] = UNSET
    if not isinstance(extent, Unset):
        json_extent = extent.value

    params["extent"] = json_extent

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/submodels/{submodel_identifier}/submodel-elements/{id_short_path}",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Result, SubmodelElement]]:
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Result.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Result.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Result.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Result.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.OK:
        response_200 = SubmodelElement.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = Result.from_dict(response.json())

        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Result, SubmodelElement]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    submodel_identifier: str,
    id_short_path: str,
    *,
    client: Union[AuthenticatedClient, Client],
    level: Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel] = GetSubmodelElementByPathSubmodelRepoLevel.DEEP,
    extent: Union[
        Unset, GetSubmodelElementByPathSubmodelRepoExtent
    ] = GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE,
) -> Response[Union[Result, SubmodelElement]]:
    """Returns a specific submodel element from the Submodel at a specified path

    Args:
        submodel_identifier (str):
        id_short_path (str):
        level (Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel]):  Default:
            GetSubmodelElementByPathSubmodelRepoLevel.DEEP.
        extent (Union[Unset, GetSubmodelElementByPathSubmodelRepoExtent]):  Default:
            GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Result, SubmodelElement]]
    """

    kwargs = _get_kwargs(
        submodel_identifier=submodel_identifier,
        id_short_path=id_short_path,
        level=level,
        extent=extent,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    submodel_identifier: str,
    id_short_path: str,
    *,
    client: Union[AuthenticatedClient, Client],
    level: Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel] = GetSubmodelElementByPathSubmodelRepoLevel.DEEP,
    extent: Union[
        Unset, GetSubmodelElementByPathSubmodelRepoExtent
    ] = GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE,
) -> Optional[Union[Result, SubmodelElement]]:
    """Returns a specific submodel element from the Submodel at a specified path

    Args:
        submodel_identifier (str):
        id_short_path (str):
        level (Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel]):  Default:
            GetSubmodelElementByPathSubmodelRepoLevel.DEEP.
        extent (Union[Unset, GetSubmodelElementByPathSubmodelRepoExtent]):  Default:
            GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Result, SubmodelElement]
    """

    return sync_detailed(
        submodel_identifier=submodel_identifier,
        id_short_path=id_short_path,
        client=client,
        level=level,
        extent=extent,
    ).parsed


async def asyncio_detailed(
    submodel_identifier: str,
    id_short_path: str,
    *,
    client: Union[AuthenticatedClient, Client],
    level: Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel] = GetSubmodelElementByPathSubmodelRepoLevel.DEEP,
    extent: Union[
        Unset, GetSubmodelElementByPathSubmodelRepoExtent
    ] = GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE,
) -> Response[Union[Result, SubmodelElement]]:
    """Returns a specific submodel element from the Submodel at a specified path

    Args:
        submodel_identifier (str):
        id_short_path (str):
        level (Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel]):  Default:
            GetSubmodelElementByPathSubmodelRepoLevel.DEEP.
        extent (Union[Unset, GetSubmodelElementByPathSubmodelRepoExtent]):  Default:
            GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Result, SubmodelElement]]
    """

    kwargs = _get_kwargs(
        submodel_identifier=submodel_identifier,
        id_short_path=id_short_path,
        level=level,
        extent=extent,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    submodel_identifier: str,
    id_short_path: str,
    *,
    client: Union[AuthenticatedClient, Client],
    level: Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel] = GetSubmodelElementByPathSubmodelRepoLevel.DEEP,
    extent: Union[
        Unset, GetSubmodelElementByPathSubmodelRepoExtent
    ] = GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE,
) -> Optional[Union[Result, SubmodelElement]]:
    """Returns a specific submodel element from the Submodel at a specified path

    Args:
        submodel_identifier (str):
        id_short_path (str):
        level (Union[Unset, GetSubmodelElementByPathSubmodelRepoLevel]):  Default:
            GetSubmodelElementByPathSubmodelRepoLevel.DEEP.
        extent (Union[Unset, GetSubmodelElementByPathSubmodelRepoExtent]):  Default:
            GetSubmodelElementByPathSubmodelRepoExtent.WITHOUTBLOBVALUE.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Result, SubmodelElement]
    """

    return (
        await asyncio_detailed(
            submodel_identifier=submodel_identifier,
            id_short_path=id_short_path,
            client=client,
            level=level,
            extent=extent,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.service_description_profiles_item import ServiceDescriptionProfilesItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="ServiceDescription")


@_attrs_define
class ServiceDescription:
    """The Description object enables servers to present their capabilities to the clients, in particular which profiles
    they implement. At least one defined profile is required. Additional, proprietary attributes might be included.
    Nevertheless, the server must not expect that a regular client understands them.

        Attributes:
            profiles (Union[Unset, List[ServiceDescriptionProfilesItem]]):
    """

    profiles: Union[Unset, List[ServiceDescriptionProfilesItem]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        profiles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.profiles, Unset):
            profiles = []
            for profiles_item_data in self.profiles:
                profiles_item = profiles_item_data.value
                profiles.append(profiles_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if profiles is not UNSET:
            field_dict["profiles"] = profiles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        profiles = []
        _profiles = d.pop("profiles", UNSET)
        for profiles_item_data in _profiles or []:
            profiles_item = ServiceDescriptionProfilesItem(profiles_item_data)

            profiles.append(profiles_item)

        service_description = cls(
            profiles=profiles,
        )

        service_description.additional_properties = d
        return service_description

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_specification_content import DataSpecificationContent
    from ..models.reference import Reference


T = TypeVar("T", bound="EmbeddedDataSpecification")


@_attrs_define
class EmbeddedDataSpecification:
    """
    Attributes:
        data_specification (Union[Unset, Reference]):
        data_specification_content (Union[Unset, DataSpecificationContent]):
    """

    data_specification: Union[Unset, "Reference"] = UNSET
    data_specification_content: Union[Unset, "DataSpecificationContent"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        data_specification: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_specification, Unset):
            data_specification = self.data_specification.to_dict()

        data_specification_content: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_specification_content, Unset):
            data_specification_content = self.data_specification_content.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if data_specification is not UNSET:
            field_dict["dataSpecification"] = data_specification
        if data_specification_content is not UNSET:
            field_dict["dataSpecificationContent"] = data_specification_content

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_specification_content import DataSpecificationContent
        from ..models.reference import Reference

        d = src_dict.copy()
        _data_specification = d.pop("dataSpecification", UNSET)
        data_specification: Union[Unset, Reference]
        if isinstance(_data_specification, Unset):
            data_specification = UNSET
        else:
            data_specification = Reference.from_dict(_data_specification)

        _data_specification_content = d.pop("dataSpecificationContent", UNSET)
        data_specification_content: Union[Unset, DataSpecificationContent]
        if isinstance(_data_specification_content, Unset):
            data_specification_content = UNSET
        else:
            data_specification_content = DataSpecificationContent.from_dict(_data_specification_content)

        embedded_data_specification = cls(
            data_specification=data_specification,
            data_specification_content=data_specification_content,
        )

        embedded_data_specification.additional_properties = d
        return embedded_data_specification

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embedded_data_specification import EmbeddedDataSpecification
    from ..models.reference import Reference


T = TypeVar("T", bound="AdministrativeInformation")


@_attrs_define
class AdministrativeInformation:
    """
    Attributes:
        version (Union[Unset, str]):
        revision (Union[Unset, str]):
        creator (Union[Unset, Reference]):
        template_id (Union[Unset, str]):
        embedded_data_specifications (Union[Unset, List['EmbeddedDataSpecification']]):
    """

    version: Union[Unset, str] = UNSET
    revision: Union[Unset, str] = UNSET
    creator: Union[Unset, "Reference"] = UNSET
    template_id: Union[Unset, str] = UNSET
    embedded_data_specifications: Union[Unset, List["EmbeddedDataSpecification"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        version = self.version

        revision = self.revision

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.creator, Unset):
            creator = self.creator.to_dict()

        template_id = self.template_id

        embedded_data_specifications: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.embedded_data_specifications, Unset):
            embedded_data_specifications = []
            for embedded_data_specifications_item_data in self.embedded_data_specifications:
                embedded_data_specifications_item = embedded_data_specifications_item_data.to_dict()
                embedded_data_specifications.append(embedded_data_specifications_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if version is not UNSET:
            field_dict["version"] = version
        if revision is not UNSET:
            field_dict["revision"] = revision
        if creator is not UNSET:
            field_dict["creator"] = creator
        if template_id is not UNSET:
            field_dict["templateId"] = template_id
        if embedded_data_specifications is not UNSET:
            field_dict["embeddedDataSpecifications"] = embedded_data_specifications

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embedded_data_specification import EmbeddedDataSpecification
        from ..models.reference import Reference

        d = src_dict.copy()
        version = d.pop("version", UNSET)

        revision = d.pop("revision", UNSET)

        _creator = d.pop("creator", UNSET)
        creator: Union[Unset, Reference]
        if isinstance(_creator, Unset):
            creator = UNSET
        else:
            creator = Reference.from_dict(_creator)

        template_id = d.pop("templateId", UNSET)

        embedded_data_specifications = []
        _embedded_data_specifications = d.pop("embeddedDataSpecifications", UNSET)
        for embedded_data_specifications_item_data in _embedded_data_specifications or []:
            embedded_data_specifications_item = EmbeddedDataSpecification.from_dict(
                embedded_data_specifications_item_data
            )

            embedded_data_specifications.append(embedded_data_specifications_item)

        administrative_information = cls(
            version=version,
            revision=revision,
            creator=creator,
            template_id=template_id,
            embedded_data_specifications=embedded_data_specifications,
        )

        administrative_information.additional_properties = d
        return administrative_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.submodel_element_list_type_value_list_element import SubmodelElementListTypeValueListElement
from ..models.submodel_element_list_value_type_list_element import SubmodelElementListValueTypeListElement
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embedded_data_specification import EmbeddedDataSpecification
    from ..models.extension import Extension
    from ..models.lang_string_name_type import LangStringNameType
    from ..models.lang_string_text_type import LangStringTextType
    from ..models.qualifier import Qualifier
    from ..models.reference import Reference
    from ..models.submodel_element import SubmodelElement


T = TypeVar("T", bound="SubmodelElementList")


@_attrs_define
class SubmodelElementList:
    """
    Attributes:
        embedded_data_specifications (Union[Unset, List['EmbeddedDataSpecification']]):
        display_name (Union[Unset, List['LangStringNameType']]):
        description (Union[Unset, List['LangStringTextType']]):
        category (Union[Unset, str]):
        id_short (Union[Unset, str]):
        extensions (Union[Unset, List['Extension']]):
        semantic_id (Union[Unset, Reference]):
        supplemental_semantic_ids (Union[Unset, List['Reference']]):
        qualifiers (Union[Unset, List['Qualifier']]):
        order_relevant (Union[Unset, bool]):
        value (Union[Unset, List['SubmodelElement']]):
        type_value_list_element (Union[Unset, SubmodelElementListTypeValueListElement]):
        value_type_list_element (Union[Unset, SubmodelElementListValueTypeListElement]):
        semantic_id_list_element (Union[Unset, Reference]):
    """

    embedded_data_specifications: Union[Unset, List["EmbeddedDataSpecification"]] = UNSET
    display_name: Union[Unset, List["LangStringNameType"]] = UNSET
    description: Union[Unset, List["LangStringTextType"]] = UNSET
    category: Union[Unset, str] = UNSET
    id_short: Union[Unset, str] = UNSET
    extensions: Union[Unset, List["Extension"]] = UNSET
    semantic_id: Union[Unset, "Reference"] = UNSET
    supplemental_semantic_ids: Union[Unset, List["Reference"]] = UNSET
    qualifiers: Union[Unset, List["Qualifier"]] = UNSET
    order_relevant: Union[Unset, bool] = UNSET
    value: Union[Unset, List["SubmodelElement"]] = UNSET
    type_value_list_element: Union[Unset, SubmodelElementListTypeValueListElement] = UNSET
    value_type_list_element: Union[Unset, SubmodelElementListValueTypeListElement] = UNSET
    semantic_id_list_element: Union[Unset, "Reference"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        embedded_data_specifications: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.embedded_data_specifications, Unset):
            embedded_data_specifications = []
            for embedded_data_specifications_item_data in self.embedded_data_specifications:
                embedded_data_specifications_item = embedded_data_specifications_item_data.to_dict()
                embedded_data_specifications.append(embedded_data_specifications_item)

        display_name: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.display_name, Unset):
            display_name = []
            for display_name_item_data in self.display_name:
                display_name_item = display_name_item_data.to_dict()
                display_name.append(display_name_item)

        description: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.description, Unset):
            description = []
            for description_item_data in self.description:
                description_item = description_item_data.to_dict()
                description.append(description_item)

        category = self.category

        id_short = self.id_short

        extensions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.extensions, Unset):
            extensions = []
            for extensions_item_data in self.extensions:
                extensions_item = extensions_item_data.to_dict()
                extensions.append(extensions_item)

        semantic_id: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.semantic_id, Unset):
            semantic_id = self.semantic_id.to_dict()

        supplemental_semantic_ids: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.supplemental_semantic_ids, Unset):
            supplemental_semantic_ids = []
            for supplemental_semantic_ids_item_data in self.supplemental_semantic_ids:
                supplemental_semantic_ids_item = supplemental_semantic_ids_item_data.to_dict()
                supplemental_semantic_ids.append(supplemental_semantic_ids_item)

        qualifiers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.qualifiers, Unset):
            qualifiers = []
            for qualifiers_item_data in self.qualifiers:
                qualifiers_item = qualifiers_item_data.to_dict()
                qualifiers.append(qualifiers_item)

        order_relevant = self.order_relevant

        value: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.value, Unset):
            value = []
            for value_item_data in self.value:
                value_item = value_item_data.to_dict()
                value.append(value_item)

        type_value_list_element: Union[Unset, str] = UNSET
        if not isinstance(self.type_value_list_element, Unset):
            type_value_list_element = self.type_value_list_element.value

        value_type_list_element: Union[Unset, str] = UNSET
        if not isinstance(self.value_type_list_element, Unset):
            value_type_list_element = self.value_type_list_element.value

        semantic_id_list_element: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.semantic_id_list_element, Unset):
            semantic_id_list_element = self.semantic_id_list_element.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if embedded_data_specifications is not UNSET:
            field_dict["embeddedDataSpecifications"] = embedded_data_specifications
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if description is not UNSET:
            field_dict["description"] = description
        if category is not UNSET:
            field_dict["category"] = category
        if id_short is not UNSET:
            field_dict["idShort"] = id_short
        if extensions is not UNSET:
            field_dict["extensions"] = extensions
        if semantic_id is not UNSET:
            field_dict["semanticId"] = semantic_id
        if supplemental_semantic_ids is not UNSET:
            field_dict["supplementalSemanticIds"] = supplemental_semantic_ids
        if qualifiers is not UNSET:
            field_dict["qualifiers"] = qualifiers
        if order_relevant is not UNSET:
            field_dict["orderRelevant"] = order_relevant
        if value is not UNSET:
            field_dict["value"] = value
        if type_value_list_element is not UNSET:
            field_dict["typeValueListElement"] = type_value_list_element
        if value_type_list_element is not UNSET:
            field_dict["valueTypeListElement"] = value_type_list_element
        if semantic_id_list_element is not UNSET:
            field_dict["semanticIdListElement"] = semantic_id_list_element

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embedded_data_specification import EmbeddedDataSpecification
        from ..models.extension import Extension
        from ..models.lang_string_name_type import LangStringNameType
        from ..models.lang_string_text_type import LangStringTextType
        from ..models.qualifier import Qualifier
        from ..models.reference import Reference
        from ..models.submodel_element import SubmodelElement

        d = src_dict.copy()
        embedded_data_specifications = []
        _embedded_data_specifications = d.pop("embeddedDataSpecifications", UNSET)
        for embedded_data_specifications_item_data in _embedded_data_specifications or []:
            embedded_data_specifications_item = EmbeddedDataSpecification.from_dict(
                embedded_data_specifications_item_data
            )

            embedded_data_specifications.append(embedded_data_specifications_item)

        display_name = []
        _display_name = d.pop("displayName", UNSET)
        for display_name_item_data in _display_name or []:
            display_name_item = LangStringNameType.from_dict(display_name_item_data)

            display_name.append(display_name_item)

        description = []
        _description = d.pop("description", UNSET)
        for description_item_data in _description or []:
            description_item = LangStringTextType.from_dict(description_item_data)

            description.append(description_item)

        category = d.pop("category", UNSET)

        id_short = d.pop("idShort", UNSET)

        extensions = []
        _extensions = d.pop("extensions", UNSET)
        for extensions_item_data in _extensions or []:
            extensions_item = Extension.from_dict(extensions_item_data)

            extensions.append(extensions_item)

        _semantic_id = d.pop("semanticId", UNSET)
        semantic_id: Union[Unset, Reference]
        if isinstance(_semantic_id, Unset):
            semantic_id = UNSET
        else:
            semantic_id = Reference.from_dict(_semantic_id)

        supplemental_semantic_ids = []
        _supplemental_semantic_ids = d.pop("supplementalSemanticIds", UNSET)
        for supplemental_semantic_ids_item_data in _supplemental_semantic_ids or []:
            supplemental_semantic_ids_item = Reference.from_dict(supplemental_semantic_ids_item_data)

            supplemental_semantic_ids.append(supplemental_semantic_ids_item)

        qualifiers = []
        _qualifiers = d.pop("qualifiers", UNSET)
        for qualifiers_item_data in _qualifiers or []:
            qualifiers_item = Qualifier.from_dict(qualifiers_item_data)

            qualifiers.append(qualifiers_item)

        order_relevant = d.pop("orderRelevant", UNSET)

        value = []
        _value = d.pop("value", UNSET)
        for value_item_data in _value or []:
            value_item = SubmodelElement.from_dict(value_item_data)

            value.append(value_item)

        _type_value_list_element = d.pop("typeValueListElement", UNSET)
        type_value_list_element: Union[Unset, SubmodelElementListTypeValueListElement]
        if isinstance(_type_value_list_element, Unset):
            type_value_list_element = UNSET
        else:
            type_value_list_element = SubmodelElementListTypeValueListElement(_type_value_list_element)

        _value_type_list_element = d.pop("valueTypeListElement", UNSET)
        value_type_list_element: Union[Unset, SubmodelElementListValueTypeListElement]
        if isinstance(_value_type_list_element, Unset):
            value_type_list_element = UNSET
        else:
            value_type_list_element = SubmodelElementListValueTypeListElement(_value_type_list_element)

        _semantic_id_list_element = d.pop("semanticIdListElement", UNSET)
        semantic_id_list_element: Union[Unset, Reference]
        if isinstance(_semantic_id_list_element, Unset):
            semantic_id_list_element = UNSET
        else:
            semantic_id_list_element = Reference.from_dict(_semantic_id_list_element)

        submodel_element_list = cls(
            embedded_data_specifications=embedded_data_specifications,
            display_name=display_name,
            description=description,
            category=category,
            id_short=id_short,
            extensions=extensions,
            semantic_id=semantic_id,
            supplemental_semantic_ids=supplemental_semantic_ids,
            qualifiers=qualifiers,
            order_relevant=order_relevant,
            value=value,
            type_value_list_element=type_value_list_element,
            value_type_list_element=value_type_list_element,
            semantic_id_list_element=semantic_id_list_element,
        )

        submodel_element_list.additional_properties = d
        return submodel_element_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class QualifierKind(str, Enum):
    CONCEPT_QUALIFIER = "CONCEPT_QUALIFIER"
    TEMPLATE_QUALIFIER = "TEMPLATE_QUALIFIER"
    VALUE_QUALIFIER = "VALUE_QUALIFIER"

    def __str__(self) -> str:
        return str(self.value)



"""Contains all the data models used in inputs/outputs"""

from .administrative_information import AdministrativeInformation
from .asset_administration_shell import AssetAdministrationShell
from .asset_information import AssetInformation
from .asset_information_asset_kind import AssetInformationAssetKind
from .base_64_url_encoded_cursor import Base64UrlEncodedCursor
from .concept_description import ConceptDescription
from .data_specification_content import DataSpecificationContent
from .embedded_data_specification import EmbeddedDataSpecification
from .entity import Entity
from .entity_entity_type import EntityEntityType
from .extension import Extension
from .extension_value_type import ExtensionValueType
from .get_all_asset_administration_shells_limit import GetAllAssetAdministrationShellsLimit
from .get_all_concept_descriptions_limit import GetAllConceptDescriptionsLimit
from .get_all_submodel_elements_extent import GetAllSubmodelElementsExtent
from .get_all_submodel_elements_level import GetAllSubmodelElementsLevel
from .get_all_submodel_elements_limit import GetAllSubmodelElementsLimit
from .get_all_submodel_references_aas_repository_limit import GetAllSubmodelReferencesAasRepositoryLimit
from .get_all_submodels_extent import GetAllSubmodelsExtent
from .get_all_submodels_level import GetAllSubmodelsLevel
from .get_all_submodels_limit import GetAllSubmodelsLimit
from .get_asset_administration_shells_result import GetAssetAdministrationShellsResult
from .get_submodel_by_id_extent import GetSubmodelByIdExtent
from .get_submodel_by_id_level import GetSubmodelByIdLevel
from .get_submodel_by_id_metadata_level import GetSubmodelByIdMetadataLevel
from .get_submodel_by_id_value_only_extent import GetSubmodelByIdValueOnlyExtent
from .get_submodel_by_id_value_only_level import GetSubmodelByIdValueOnlyLevel
from .get_submodel_element_by_path_submodel_repo_extent import GetSubmodelElementByPathSubmodelRepoExtent
from .get_submodel_element_by_path_submodel_repo_level import GetSubmodelElementByPathSubmodelRepoLevel
from .get_submodel_element_by_path_value_only_submodel_repo_extent import (
    GetSubmodelElementByPathValueOnlySubmodelRepoExtent,
)
from .get_submodel_element_by_path_value_only_submodel_repo_level import (
    GetSubmodelElementByPathValueOnlySubmodelRepoLevel,
)
from .key import Key
from .key_type import KeyType
from .lang_string_name_type import LangStringNameType
from .lang_string_text_type import LangStringTextType
from .message import Message
from .message_message_type import MessageMessageType
from .operation import Operation
from .operation_request import OperationRequest
from .operation_request_client_timeout_duration import OperationRequestClientTimeoutDuration
from .operation_request_client_timeout_duration_xmlschema_type import OperationRequestClientTimeoutDurationXmlschemaType
from .operation_variable import OperationVariable
from .paged_result_paging_metadata import PagedResultPagingMetadata
from .patch_submodel_by_id_value_only_level import PatchSubmodelByIdValueOnlyLevel
from .patch_submodel_element_by_path_value_only_submodel_repo_level import (
    PatchSubmodelElementByPathValueOnlySubmodelRepoLevel,
)
from .post_submodel_element_by_path_submodel_repo_extent import PostSubmodelElementByPathSubmodelRepoExtent
from .post_submodel_element_by_path_submodel_repo_level import PostSubmodelElementByPathSubmodelRepoLevel
from .put_file_by_path_body import PutFileByPathBody
from .put_submodel_by_id_level import PutSubmodelByIdLevel
from .put_submodel_element_by_path_submodel_repo_level import PutSubmodelElementByPathSubmodelRepoLevel
from .put_thumbnail_aas_repository_body import PutThumbnailAasRepositoryBody
from .qualifier import Qualifier
from .qualifier_kind import QualifierKind
from .qualifier_value_type import QualifierValueType
from .reference import Reference
from .reference_type import ReferenceType
from .relationship_element import RelationshipElement
from .resource import Resource
from .result import Result
from .service_description import ServiceDescription
from .service_description_profiles_item import ServiceDescriptionProfilesItem
from .specific_asset_id import SpecificAssetId
from .submodel import Submodel
from .submodel_element import SubmodelElement
from .submodel_element_collection import SubmodelElementCollection
from .submodel_element_list import SubmodelElementList
from .submodel_element_list_type_value_list_element import SubmodelElementListTypeValueListElement
from .submodel_element_list_value_type_list_element import SubmodelElementListValueTypeListElement
from .submodel_element_value import SubmodelElementValue
from .submodel_kind import SubmodelKind
from .submodel_value_only import SubmodelValueOnly
from .submodel_value_only_values_only_map import SubmodelValueOnlyValuesOnlyMap
from .upload_environment_body import UploadEnvironmentBody

__all__ = (
    "AdministrativeInformation",
    "AssetAdministrationShell",
    "AssetInformation",
    "AssetInformationAssetKind",
    "Base64UrlEncodedCursor",
    "ConceptDescription",
    "DataSpecificationContent",
    "EmbeddedDataSpecification",
    "Entity",
    "EntityEntityType",
    "Extension",
    "ExtensionValueType",
    "GetAllAssetAdministrationShellsLimit",
    "GetAllConceptDescriptionsLimit",
    "GetAllSubmodelElementsExtent",
    "GetAllSubmodelElementsLevel",
    "GetAllSubmodelElementsLimit",
    "GetAllSubmodelReferencesAasRepositoryLimit",
    "GetAllSubmodelsExtent",
    "GetAllSubmodelsLevel",
    "GetAllSubmodelsLimit",
    "GetAssetAdministrationShellsResult",
    "GetSubmodelByIdExtent",
    "GetSubmodelByIdLevel",
    "GetSubmodelByIdMetadataLevel",
    "GetSubmodelByIdValueOnlyExtent",
    "GetSubmodelByIdValueOnlyLevel",
    "GetSubmodelElementByPathSubmodelRepoExtent",
    "GetSubmodelElementByPathSubmodelRepoLevel",
    "GetSubmodelElementByPathValueOnlySubmodelRepoExtent",
    "GetSubmodelElementByPathValueOnlySubmodelRepoLevel",
    "Key",
    "KeyType",
    "LangStringNameType",
    "LangStringTextType",
    "Message",
    "MessageMessageType",
    "Operation",
    "OperationRequest",
    "OperationRequestClientTimeoutDuration",
    "OperationRequestClientTimeoutDurationXmlschemaType",
    "OperationVariable",
    "PagedResultPagingMetadata",
    "PatchSubmodelByIdValueOnlyLevel",
    "PatchSubmodelElementByPathValueOnlySubmodelRepoLevel",
    "PostSubmodelElementByPathSubmodelRepoExtent",
    "PostSubmodelElementByPathSubmodelRepoLevel",
    "PutFileByPathBody",
    "PutSubmodelByIdLevel",
    "PutSubmodelElementByPathSubmodelRepoLevel",
    "PutThumbnailAasRepositoryBody",
    "Qualifier",
    "QualifierKind",
    "QualifierValueType",
    "Reference",
    "ReferenceType",
    "RelationshipElement",
    "Resource",
    "Result",
    "ServiceDescription",
    "ServiceDescriptionProfilesItem",
    "SpecificAssetId",
    "Submodel",
    "SubmodelElement",
    "SubmodelElementCollection",
    "SubmodelElementList",
    "SubmodelElementListTypeValueListElement",
    "SubmodelElementListValueTypeListElement",
    "SubmodelElementValue",
    "SubmodelKind",
    "SubmodelValueOnly",
    "SubmodelValueOnlyValuesOnlyMap",
    "UploadEnvironmentBody",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.operation_request_client_timeout_duration_xmlschema_type import (
        OperationRequestClientTimeoutDurationXmlschemaType,
    )


T = TypeVar("T", bound="OperationRequestClientTimeoutDuration")


@_attrs_define
class OperationRequestClientTimeoutDuration:
    """
    Attributes:
        sign (Union[Unset, int]):
        xmlschema_type (Union[Unset, OperationRequestClientTimeoutDurationXmlschemaType]):
        months (Union[Unset, int]):
        seconds (Union[Unset, int]):
        days (Union[Unset, int]):
        years (Union[Unset, int]):
        hours (Union[Unset, int]):
        minutes (Union[Unset, int]):
    """

    sign: Union[Unset, int] = UNSET
    xmlschema_type: Union[Unset, "OperationRequestClientTimeoutDurationXmlschemaType"] = UNSET
    months: Union[Unset, int] = UNSET
    seconds: Union[Unset, int] = UNSET
    days: Union[Unset, int] = UNSET
    years: Union[Unset, int] = UNSET
    hours: Union[Unset, int] = UNSET
    minutes: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sign = self.sign

        xmlschema_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.xmlschema_type, Unset):
            xmlschema_type = self.xmlschema_type.to_dict()

        months = self.months

        seconds = self.seconds

        days = self.days

        years = self.years

        hours = self.hours

        minutes = self.minutes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if sign is not UNSET:
            field_dict["sign"] = sign
        if xmlschema_type is not UNSET:
            field_dict["xmlschemaType"] = xmlschema_type
        if months is not UNSET:
            field_dict["months"] = months
        if seconds is not UNSET:
            field_dict["seconds"] = seconds
        if days is not UNSET:
            field_dict["days"] = days
        if years is not UNSET:
            field_dict["years"] = years
        if hours is not UNSET:
            field_dict["hours"] = hours
        if minutes is not UNSET:
            field_dict["minutes"] = minutes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.operation_request_client_timeout_duration_xmlschema_type import (
            OperationRequestClientTimeoutDurationXmlschemaType,
        )

        d = src_dict.copy()
        sign = d.pop("sign", UNSET)

        _xmlschema_type = d.pop("xmlschemaType", UNSET)
        xmlschema_type: Union[Unset, OperationRequestClientTimeoutDurationXmlschemaType]
        if isinstance(_xmlschema_type, Unset):
            xmlschema_type = UNSET
        else:
            xmlschema_type = OperationRequestClientTimeoutDurationXmlschemaType.from_dict(_xmlschema_type)

        months = d.pop("months", UNSET)

        seconds = d.pop("seconds", UNSET)

        days = d.pop("days", UNSET)

        years = d.pop("years", UNSET)

        hours = d.pop("hours", UNSET)

        minutes = d.pop("minutes", UNSET)

        operation_request_client_timeout_duration = cls(
            sign=sign,
            xmlschema_type=xmlschema_type,
            months=months,
            seconds=seconds,
            days=days,
            years=years,
            hours=hours,
            minutes=minutes,
        )

        operation_request_client_timeout_duration.additional_properties = d
        return operation_request_client_timeout_duration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.asset_information_asset_kind import AssetInformationAssetKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.resource import Resource
    from ..models.specific_asset_id import SpecificAssetId


T = TypeVar("T", bound="AssetInformation")


@_attrs_define
class AssetInformation:
    """
    Attributes:
        asset_kind (Union[Unset, AssetInformationAssetKind]):
        asset_type (Union[Unset, str]):
        default_thumbnail (Union[Unset, Resource]):
        global_asset_id (Union[Unset, str]):
        specific_asset_ids (Union[Unset, List['SpecificAssetId']]):
    """

    asset_kind: Union[Unset, AssetInformationAssetKind] = UNSET
    asset_type: Union[Unset, str] = UNSET
    default_thumbnail: Union[Unset, "Resource"] = UNSET
    global_asset_id: Union[Unset, str] = UNSET
    specific_asset_ids: Union[Unset, List["SpecificAssetId"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        asset_kind: Union[Unset, str] = UNSET
        if not isinstance(self.asset_kind, Unset):
            asset_kind = self.asset_kind.value

        asset_type = self.asset_type

        default_thumbnail: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_thumbnail, Unset):
            default_thumbnail = self.default_thumbnail.to_dict()

        global_asset_id = self.global_asset_id

        specific_asset_ids: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.specific_asset_ids, Unset):
            specific_asset_ids = []
            for specific_asset_ids_item_data in self.specific_asset_ids:
                specific_asset_ids_item = specific_asset_ids_item_data.to_dict()
                specific_asset_ids.append(specific_asset_ids_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if asset_kind is not UNSET:
            field_dict["assetKind"] = asset_kind
        if asset_type is not UNSET:
            field_dict["assetType"] = asset_type
        if default_thumbnail is not UNSET:
            field_dict["defaultThumbnail"] = default_thumbnail
        if global_asset_id is not UNSET:
            field_dict["globalAssetId"] = global_asset_id
        if specific_asset_ids is not UNSET:
            field_dict["specificAssetIds"] = specific_asset_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.resource import Resource
        from ..models.specific_asset_id import SpecificAssetId

        d = src_dict.copy()
        _asset_kind = d.pop("assetKind", UNSET)
        asset_kind: Union[Unset, AssetInformationAssetKind]
        if isinstance(_asset_kind, Unset):
            asset_kind = UNSET
        else:
            asset_kind = AssetInformationAssetKind(_asset_kind)

        asset_type = d.pop("assetType", UNSET)

        _default_thumbnail = d.pop("defaultThumbnail", UNSET)
        default_thumbnail: Union[Unset, Resource]
        if isinstance(_default_thumbnail, Unset):
            default_thumbnail = UNSET
        else:
            default_thumbnail = Resource.from_dict(_default_thumbnail)

        global_asset_id = d.pop("globalAssetId", UNSET)

        specific_asset_ids = []
        _specific_asset_ids = d.pop("specificAssetIds", UNSET)
        for specific_asset_ids_item_data in _specific_asset_ids or []:
            specific_asset_ids_item = SpecificAssetId.from_dict(specific_asset_ids_item_data)

            specific_asset_ids.append(specific_asset_ids_item)

        asset_information = cls(
            asset_kind=asset_kind,
            asset_type=asset_type,
            default_thumbnail=default_thumbnail,
            global_asset_id=global_asset_id,
            specific_asset_ids=specific_asset_ids,
        )

        asset_information.additional_properties = d
        return asset_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ExtensionValueType(str, Enum):
    ANY_URI = "ANY_URI"
    BASE64BINARY = "BASE64BINARY"
    BOOLEAN = "BOOLEAN"
    BYTE = "BYTE"
    DATE = "DATE"
    DATE_TIME = "DATE_TIME"
    DECIMAL = "DECIMAL"
    DOUBLE = "DOUBLE"
    DURATION = "DURATION"
    FLOAT = "FLOAT"
    GDAY = "GDAY"
    GMONTH = "GMONTH"
    GMONTH_DAY = "GMONTH_DAY"
    GYEAR = "GYEAR"
    GYEAR_MONTH = "GYEAR_MONTH"
    HEX_BINARY = "HEX_BINARY"
    INT = "INT"
    INTEGER = "INTEGER"
    LONG = "LONG"
    NEGATIVE_INTEGER = "NEGATIVE_INTEGER"
    NON_NEGATIVE_INTEGER = "NON_NEGATIVE_INTEGER"
    NON_POSITIVE_INTEGER = "NON_POSITIVE_INTEGER"
    POSITIVE_INTEGER = "POSITIVE_INTEGER"
    SHORT = "SHORT"
    STRING = "STRING"
    TIME = "TIME"
    UNSIGNED_BYTE = "UNSIGNED_BYTE"
    UNSIGNED_INT = "UNSIGNED_INT"
    UNSIGNED_LONG = "UNSIGNED_LONG"
    UNSIGNED_SHORT = "UNSIGNED_SHORT"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.message_message_type import MessageMessageType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Message")


@_attrs_define
class Message:
    """
    Attributes:
        timestamp (Union[Unset, str]):
        text (Union[Unset, str]):
        message_type (Union[Unset, MessageMessageType]):
        correlation_id (Union[Unset, str]):
        code (Union[Unset, str]):
    """

    timestamp: Union[Unset, str] = UNSET
    text: Union[Unset, str] = UNSET
    message_type: Union[Unset, MessageMessageType] = UNSET
    correlation_id: Union[Unset, str] = UNSET
    code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        timestamp = self.timestamp

        text = self.text

        message_type: Union[Unset, str] = UNSET
        if not isinstance(self.message_type, Unset):
            message_type = self.message_type.value

        correlation_id = self.correlation_id

        code = self.code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if text is not UNSET:
            field_dict["text"] = text
        if message_type is not UNSET:
            field_dict["messageType"] = message_type
        if correlation_id is not UNSET:
            field_dict["correlationId"] = correlation_id
        if code is not UNSET:
            field_dict["code"] = code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        timestamp = d.pop("timestamp", UNSET)

        text = d.pop("text", UNSET)

        _message_type = d.pop("messageType", UNSET)
        message_type: Union[Unset, MessageMessageType]
        if isinstance(_message_type, Unset):
            message_type = UNSET
        else:
            message_type = MessageMessageType(_message_type)

        correlation_id = d.pop("correlationId", UNSET)

        code = d.pop("code", UNSET)

        message = cls(
            timestamp=timestamp,
            text=text,
            message_type=message_type,
            correlation_id=correlation_id,
            code=code,
        )

        message.additional_properties = d
        return message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
